{---------------------------------------------------------------------------------------

    The purpose of this component is to allow applications to easily
    incorporate a name field that will parse First, Middle, Last, Greeting, and Suffix
    easily without having to write any code.

    This component is used just like any edit component, with the addition of
    mapping output to other edit controls.

    To parse the names from the field, the control must first have a reference to
    five other standard TEdit controls.  These must be referenced according to the
    type of data you wish for them to contain.  The fields to reference are as follows:

    GreetingParsedField

    FirstNameParsedField

    MiddleNameParsedField

    LastNameParsedField

    SuffixParsedField

    If the GreetingParser or SuffixParser does not work on every case, you may add cases
    to the StringList directly after inserting the component, or you may dynamically
    load the data from a flat text file (Formatted like a TStringList) at runtime
    for extra functionality.

---------------------------------------------------------------------------------------}
                              {Copyright  1996,1997 by Jim Wood}

unit NameEdit;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Menus,
  Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, DsgnIntf, DBCtrls;

type
  TNameEdit = class(TEdit)
  private
     FAbout: TStringList;
     FAutoFormat : boolean;
     FStripSpaces : boolean;
     FRemoveSuffix : boolean;
     FRemoveGreeting : boolean;
     FFirstNameParsedField : TEdit;
     FLastNameParsedField : TEdit;
     FGreetingParsedField : TEdit;
     FSuffixParsedField : TEdit;
     FMiddleNameParsedField : TEdit;
     FGreetingParser : TStringList;
     FSuffixParser : TStringList;
     function GetEditText: string;
     procedure SetEditText(const Value: string);
     procedure SetGreetings(Value:TStringList);
     procedure SetSuffixs(Value:TStringList);
     procedure SetAbout(Value:TStringList);
  protected
     function  StripLeadingSpaces(var StripString: string): string;
     function  StripExtraSpaces(var StripString: string): string;
     procedure KeyDown(var Key: Word; Shift: TShiftState); override;
     procedure KeyPress( var Key: char ); override;
     procedure Change; override;
     procedure GetSel(var SelStart: Integer; var SelStop: Integer);
     procedure SetSel(SelStart: Integer; SelStop: Integer);
     procedure CheckCursor;
  public
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    function GetTextLen: Integer;
    procedure ParseName;
  published
    property About: TStringList read FAbout write SetAbout;
    property AutoFormat : boolean read FAutoFormat write FAutoFormat ;
    property StripSpaces : boolean read FStripSpaces write FStripSpaces ;
    property RemoveSuffix : boolean read FRemoveSuffix write FRemoveSuffix ;
    property RemoveGreeting : boolean read FRemoveGreeting write FRemoveGreeting ;
    property FirstNameParsedField : TEdit read FFirstNameParsedField write FFirstNameParsedField;
    property LastNameParsedField : TEdit read FLastNameParsedField write FLastNameParsedField;
    property GreetingParsedField : TEdit read FGreetingParsedField write FGreetingParsedField;
    property MiddleNameParsedField : TEdit read FMiddleNameParsedField write FMiddleNameParsedField;
    property SuffixParsedField : TEdit read FSuffixParsedField write FSuffixParsedField;
    property GreetingParser : TStringList read FGreetingParser write SetGreetings ;
    property SuffixParser : TStringList read FSuffixParser write SetSuffixs ;
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Height;
    property HelpContext;
    property HideSelection;
    property Hint;
    property Left;
    property MaxLength;
    property Name;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Tag;
    property Text;
    property Top;
    property Visible;
    property Width;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
end;

var
EditField,
SetUpCaseSelection: boolean;
OldSelStart,
MoveStringBack,
CharNamePos,
CharNameLength,
CharNameStop,
CharNamePos2,
CharNameLength2,
CharNameStop2,
CheckErr: integer;

procedure Register;

implementation

{================================================================================}

procedure Register;
begin
  RegisterComponents('Additional',[TNameEdit]);
end;
 
{================================================================================}

procedure TNameEdit.KeyPress( var Key: Char );
begin
 EditField := true;
 GetSel(CharNamePos, CharNameStop);
 CharNameLength := CharNameStop-CharNamePos;
 inherited KeyPress(Key);
end;

{================================================================================}

procedure TNameEdit.GetSel(var SelStart: Integer; var SelStop: Integer);
var
  Res: LongInt;
begin
  Res := SendMessage(Handle, EM_GETSEL, 0, 0);
  SelStart := LoWord(Res);
  SelStop  := HiWord(Res);
end;

{================================================================================}

procedure TNameEdit.SetSel(SelStart: Integer; SelStop: Integer);
begin
  SendMessage(Handle, EM_SETSEL, SelStart, SelStop);
end;

{================================================================================}

procedure TNameEdit.CheckCursor;
begin
  if not HandleAllocated then  Exit;
end;

{================================================================================}

procedure TNameEdit.SetEditText(const Value: string);
var
  Buffer: array[0..255] of Char;
begin
  if GetEditText <> Value then
  begin
    SetTextBuf(StrPCopy(Buffer, Value));
    CheckCursor;
  end;
end;

{================================================================================}

function TNameEdit.GetEditText: string;
begin
  Result := inherited Text;
end;

{================================================================================}

function TNameEdit.GetTextLen: Integer;
begin
  Result := Length(Text);
end;

{================================================================================}

procedure TNameEdit.Change;
var
    Name2, Name, TempName: array [0..255] of Char;
    x,
    z: integer;
    GetText,
    PutText: String;
begin
 if FAutoFormat = true then
  begin
  try
  GetSel(CharNamePos2, CharNameStop2);
  except
    Exit;
  end;
  GetText := GetEditText;
  if Length(GetText) < 3 then GetText := StripLeadingSpaces(GetText);
  CharNameLength2 := CharNameStop2-CharNamePos;
   if (CharNamePos2 = 0) and (CharNameLength2 = 0) then Exit;
    Fillchar(Name, Sizeof(Name), #0);
    StrPCopy(Name, GetText);
    if Name[0] = #0
      then
        Exit;
    if Name[0] <> Upcase(Name[0])
      then
        begin
          Name[0] := Upcase(Name[0]);
          PutText := StrPas(Name);
          SetEditText(StripLeadingSpaces(PutText));
          if SetUpCaseSelection = false then SetSel(1,1)
          else SetSel(0,0);
          SetUpCaseSelection := false;
        end;
    if (Strlen(Name) > 2)
      then
        begin
          if (Name[Strlen(Name) - 1] = ' ') and (Name[Strlen(Name) - 2] = ',') and (SetUpCaseSelection = false)
            then
              begin
                Strcopy(Name2, ' ');
                Strlcat(Name2, Name, Strlen(Name) - 1);
                PutText := StrPas(Name2);
                SetEditText(PutText);
                SetSel(0,0);
              end;
          if (Name[Strlen(Name) - 1] <> ' ') and (Name[Strlen(Name) - 2] = ',') and (SetUpCaseSelection = false)
            then
              begin
                StrCopy(Name2, ' ');
                Name2[0] := Name[StrLen(Name) - 1];
                Strcat(Name2, ' ');
                Strlcat(Name2, Name, Strlen(Name));
                PutText := StrPas(Name2);
                SetEditText(PutText);
                SetSel(1,1);
              end;
          if ((Name[StrLen(Name) - 3] = '-')
              and (Name[StrLen(Name) - 2] <> UpCase(Name[StrLen(Name) - 2])))
              or  (Name[Strlen(Name) - 3] = ' ')
              and (Name[Strlen(Name) - 2] <> Upcase(Name[Strlen(Name) - 2]))
              or  ((Name[Strlen(Name) - 3] = '''')
              and (Name[StrLen(Name) - 5] = ' ')
              and (Name[Strlen(Name) - 2] <> Upcase(Name[Strlen(Name) -2])))
              or  ((Name[StrLen(Name) - 5] = ' ')
              and (Upcase(Name[Strlen(Name) - 3]) = 'C')
              and (Upcase(Name[Strlen(Name) - 4]) = 'M')
              and (Name[Strlen(Name) - 1] <> ' ')
              and (Name[Strlen(Name) - 2] <> Upcase(Name[Strlen(Name) - 2])))
              or  ((Name[StrLen(Name) - 6] = ' ')
              and (Upcase(Name[Strlen(Name) - 3]) = 'C')
              and (Upcase(Name[Strlen(Name) - 4]) = 'A')
              and (Upcase(Name[Strlen(Name) - 5]) = 'M')
              and (Name[Strlen(Name) - 1] <> ' ')
              and (Name[Strlen(Name) - 2] <> Upcase(Name[Strlen(Name) - 2])))
            then
              begin
                Name[Strlen(Name) - 2] := Upcase(Name[Strlen(Name) - 2]);
                PutText := StrPas(Name);
                SetEditText(StripLeadingSpaces(PutText));
                SetSel(StrLen(Name), StrLen(Name));
              end;
          if ((Name[CharNamePos2 - 2] = ' ') and (Name[CharNamePos2 - 1] <>
              UpCase(Name[CharNamePos2 - 1])) or
              (Name[CharNamePos2 - 2] = '/') and (Name[CharNamePos2 - 1] <>
              UpCase(Name[CharNamePos2 - 1])) or
              (Name[CharNamePos2 - 2] = '.') and (Name[CharNamePos2 - 1] <>
              UpCase(Name[CharNamePos2 - 1])) or
              (Name[CharNamePos2 - 2] = '''') and
              (Name[CharNamePos2 - 4] = ' ') and
              (Name[CharNamePos2 - 1] <> UpCase(Name[CharNamePos2 - 1])) or
              (Name[CharNamePos2 - 2] = '`') and (Name[CharNamePos2 - 1] <>
              UpCase(Name[CharNamePos2 - 1])))
            then
              begin
                Name[CharNamePos2 - 1] := Upcase(Name[CharNamePos2 - 1]);
                OldSelStart := CharNamePos2;
                PutText := StrPas(Name);
                SetEditText(StripLeadingSpaces(PutText));
                SetSel(OldSelStart,OldSelStart);
              end;
          if ((Name[StrLen(Name)-2] <> 'M') and (Name[StrLen(Name)- 3] = ' ') and
              (Name[StrLen(Name) - 4] = 's') and (Name[StrLen(Name) - 5] = 'e') and
              (Name[StrLen(Name) - 6] = 'D') and (Name[StrLen(Name) - 7] = ' ')) then
               begin
                Name[StrLen(Name) - 6] := 'd';
                PutText := StrPas(Name);
                SetEditText(StripLeadingSpaces(PutText));
                SetSel(StrLen(Name), StrLen(Name));
               end;
          if ((Name[StrLen(Name) - 2] = ' ') and (Name[StrLen(Name) - 3] = 'e') and
              (Name[StrLen(Name) - 4] = 'D') and (Name[StrLen(Name) - 5] = ' ')) then
               begin
                Name[StrLen(Name) - 4] := 'd';
                PutText := StrPas(Name);
                SetEditText(StripLeadingSpaces(PutText));
                SetSel(StrLen(Name), StrLen(Name));
               end;
          if ((Name[StrLen(Name) - 2] = ' ') and (Name[StrLen(Name) - 3] = 'u') and
              (Name[StrLen(Name) - 4] = 'D') and (Name[StrLen(Name) - 5] = ' ')) then
               begin
                Name[StrLen(Name) - 4] := 'd';
                PutText := StrPas(Name);
                SetEditText(StripLeadingSpaces(PutText));
                SetSel(StrLen(Name), StrLen(Name));
               end;

          {Begin Suffix Parser Routine}
          for z := 0 to FSuffixParser.Count-1 do
           begin
             if Length(FSuffixParser[z]) = 2 then
            begin
             if ((Upcase(Name[1]) = Upcase(FSuffixParser[z][2])) and
                 (Upcase(Name[0]) = Upcase(FSuffixParser[z][1])))
             then
              begin
               StrCat(Name, ', ');
               StrPCopy(TempName, FSuffixParser[z]);
               StrCat(Name, TempName);
               for x := 0 to StrLen(Name) - 3 do
               Name[x] := Name[x + 3];
               PutText := StrPas(Name);
               SetEditText(StripLeadingSpaces(PutText));
               SetSel(StrLen(Name), StrLen(Name));
              end;
            end
           else
            if Length(FSuffixParser[z]) = 3 then
            begin
             if ((Upcase(Name[2]) = Upcase(FSuffixParser[z][3])) and
                 (Upcase(Name[1]) = Upcase(FSuffixParser[z][2])) and
                 (Upcase(Name[0]) = Upcase(FSuffixParser[z][1])))
             then
              begin
               StrCat(Name, ', ');
               StrPCopy(TempName, FSuffixParser[z]);
               StrCat(Name, TempName);
               for x := 0 to StrLen(Name) - 4 do
               Name[x] := Name[x + 4];
               PutText := StrPas(Name);
               SetEditText(StripLeadingSpaces(PutText));
               SetSel(StrLen(Name), StrLen(Name));
              end;
            end
           else
            if Length(FSuffixParser[z]) = 4 then
            begin
             if ((Upcase(Name[3]) = Upcase(FSuffixParser[z][4])) and
                 (Upcase(Name[2]) = Upcase(FSuffixParser[z][3])) and
                 (Upcase(Name[1]) = Upcase(FSuffixParser[z][2])) and
                 (Upcase(Name[0]) = Upcase(FSuffixParser[z][1])))
             then
              begin
               StrCat(Name, ', ');
               StrPCopy(TempName, FSuffixParser[z]);
               StrCat(Name, TempName);
               for x := 0 to StrLen(Name) - 5 do
               Name[x] := Name[x + 5];
               PutText := StrPas(Name);
               SetEditText(StripLeadingSpaces(PutText));
               SetSel(StrLen(Name), StrLen(Name));
              end;
            end;
           end;
          {End of Suffix Parser Routine}

        end;
  end;
  inherited Change;
end;

{================================================================================}

procedure TNameEdit.KeyDown(var Key: Word; Shift: TShiftState);
var
    Name3: array [0..40] of Char;
begin
 SetUpCaseSelection := false;
 if Key = VK_BACK then
  begin
   SetUpCaseSelection := true;
  end;
 if Key = VK_DELETE then
  begin
   SetUpCaseSelection := true;
  end;
 if Key = VK_UP then
  begin
  end;
 if (Key = VK_RETURN) or (Key = VK_DOWN) then
  begin
   StrPCopy(Name3, GetEditText);
   if StrComp(Name3, '') = 0 then
    begin
     {Make Noise?}
     {MessageBeep(0);}
    end;
   {Send Message to user if return is pressed}
  end;
 inherited KeyDown(Key, Shift);
end;

{================================================================================}

procedure TNameEdit.ParseName;
var
  SplitString1,
  SplitString2: String;
  SpacePos,
  z: integer;
  ShowCredits,
  RemoveGreeting: boolean;
begin
 {Insert Parsing Routine here.}
   if Assigned(FFirstNameParsedField) then FFirstNameParsedField.Text := '';
   if Assigned(FLastNameParsedField) then FLastNameParsedField.Text := '';
   if Assigned(FMiddleNameParsedField) then FMiddleNameParsedField.Text := '';
   if Assigned(FGreetingParsedField) then FGreetingParsedField.Text := '';
   if Assigned(FSuffixParsedField) then FSuffixParsedField.Text := '';
   SplitString1 := GetEditText;
   SplitString1 := StripExtraSpaces(SplitString1);
   if Length(SplitString1) = 0 then
    begin
     Exit;
    end;
   ShowCredits := false;
   if Pos('Show Credits', SplitString1) > 0 then ShowCredits := true;
   SpacePos := Pos(' ', SplitString1);
   SplitString2 := Copy(SplitString1, 0, SpacePos-1);
   if ShowCredits = true then
    begin
     MessageBox(Handle, 'Thank you for using the TNameEdit component Written by: Jim Wood', 'TNameEdit Component', mb_OK);
    end;

   {Begin Greeting Parser}
   RemoveGreeting := false;
   for z := 0 to FGreetingParser.Count-1 do
    begin
     if AnsiCompareText(SplitString2, FGreetingParser[z]) = 0 then RemoveGreeting := true;
    end;
   if RemoveGreeting = true then
    begin
     SplitString2 := Copy(SplitString1, 0, SpacePos-1);
     if Assigned(FGreetingParsedField) then FGreetingParsedField.Text := SplitString2;
     SplitString2 := Copy(SplitString1, SpacePos+1, Length(SplitString1));
     SplitString1 := SplitString2;
     if FRemoveGreeting = true then SetEditText(StripLeadingSpaces(SplitString1));
     SpacePos := Pos(' ', SplitString1);
     SplitString2 := Copy(SplitString1, 0, SpacePos-1);
    end;
   {End Greeting Parser}

   if Assigned(FFirstNameParsedField) then FFirstNameParsedField.Text := SplitString2;
   SplitString2 := Copy(SplitString1, SpacePos+1, Length(SplitString1));
   SpacePos := Pos('. ', SplitString2);
   if SpacePos > 0 then
    begin
     SplitString1 := Copy(SplitString2, 0, SpacePos);
     if Assigned(FMiddleNameParsedField) then FMiddleNameParsedField.Text := SplitString1;
     SplitString1 := Copy(SplitString2, SpacePos+2, Length(SplitString2));
     SplitString2 := SplitString1;
    end
   else
    begin
     SpacePos := Pos(' ', SplitString2);
     if SpacePos > 0 then
     begin
      SplitString1 := Copy(SplitString2, 0, SpacePos-1);
      if Assigned(FMiddleNameParsedField) then FMiddleNameParsedField.Text := SplitString1;
      SplitString1 := Copy(SplitString2, SpacePos+1, Length(SplitString2));
      SplitString2 := SplitString1;
     end;
    end;
   SpacePos := Pos(',', SplitString2);
   if SpacePos = 0 then
   begin
    SpacePos := Pos('/', SplitString2);
    if SpacePos > 0 then
     begin
      SplitString1 := Copy(SplitString2, 0, SpacePos-1);
      SplitString2 := SplitString1;
     end
    else
     begin
     SpacePos := Pos(' ', SplitString2);
     if (SpacePos > 0) then
      begin
       SplitString1 := Copy(SplitString2, 0, SpacePos);
       if Assigned(FMiddleNameParsedField) then FMiddleNameParsedField.Text := SplitString1;
       SplitString1 := Copy(SplitString2, SpacePos+1, Length(SplitString2));
       SplitString2 := SplitString1;
      end
     else
      begin
       SpacePos := Pos(' ', SplitString2);
       if SpacePos > 0 then
       SplitString1 := Copy(SplitString2, SpacePos+1, Length(SplitString2))
       else SplitString1 := SplitString2;
      end;
     end;
   end
  else
   begin
    SplitString1 := Copy(Splitstring2, SpacePos+2, Length(SplitString2)-SpacePos+2);
    if Assigned(FSuffixParsedField) then FSuffixParsedField.Text := SplitString1;
    SplitString1 := Copy(SplitString2, 0, SpacePos-1);
    SplitString2 := SplitString1;
   end;
  SplitString1 := SplitString1;
  if Assigned(FLastNameParsedField) then FLastNameParsedField.Text := SplitString1;
  SplitString1 := GetEditText;
  SpacePos := Pos(',', SplitString1);
  if SpacePos > 0 then
  begin
   SplitString2 := Copy(SplitString1, 0, SpacePos-1);
   if FRemoveSuffix = true then SetEditText(StripLeadingSpaces(SplitString2));
  end;
end;

{================================================================================}

function TNameEdit.StripExtraSpaces(var StripString: string): string;
var
 StripPos,
 StripExtra: integer;
 TempStrip1,
 TempStrip2: string;
begin
 if FStripSpaces = true then
 begin
  for StripPos := 1 to Length(StripString) do
   begin
    StripExtra := Pos('  ', StripString);
    if StripExtra > 0 then
     begin
      TempStrip1 := Copy(StripString, 0, StripExtra-1);
      TempStrip2 := Copy(StripString, StripExtra+1, Length(StripString)-StripExtra+1);
      StripString := TempStrip1 + TempStrip2;
     end;
   end;
 end;
 Result := StripString;
end;

{================================================================================}

function TNameEdit.StripLeadingSpaces(var StripString: string): string;
var
 CopyFromPos,
 StripPos: integer;
 TempStrip: string;
 StopHere: Boolean;
begin
 if FStripSpaces = true then
 begin
  CopyFromPos := 0;
  StopHere := false;
  for StripPos := 1 to Length(StripString) do
   begin
    if (StopHere = false) and (StripString[StripPos] = ' ') then inc(CopyFromPos) else StopHere := true;
   end;
  TempStrip := Copy(StripString, CopyFromPos+1, Length(StripString)-CopyFromPos+1);
  Result := TempStrip;
 end
 else
 Result := StripString;
end;

{================================================================================}

constructor TNameEdit.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
     FAutoFormat := TRUE;
     FRemoveSuffix := FALSE;
     FRemoveGreeting := FALSE;
     FStripSpaces := TRUE;
     FGreetingParser := TStringList.Create;
     FSuffixParser := TStringList.Create;
     FAbout := TStringList.Create;

     {Create default greeting list}
     FAbout.Clear;
     FAbout.Add('TNameEdit - Version 1.2');
     FAbout.Add('-----------------------------------------');
     FAbout.Add(' ');
     FAbout.Add('This component is distributed as freeware.');
     FAbout.Add(' ');
     FAbout.Add('E-Mail comments or questions to: jwood@itp-cti.com');

     {Create default greeting list}
     FGreetingParser.Clear;
     FGreetingParser.Add('Mr.');
     FGreetingParser.Add('Ms.');
     FGreetingParser.Add('Miss.');
     FGreetingParser.Add('Mrs.');
     FGreetingParser.Add('Dr.');
     FGreetingParser.Add('Mr');
     FGreetingParser.Add('Ms');
     FGreetingParser.Add('Miss');
     FGreetingParser.Add('Mrs');
     FGreetingParser.Add('Dr.');
     FGreetingParser.Add('Fr.');
     FGreetingParser.Add('Capt.');
     FGreetingParser.Add('Lt.');
     FGreetingParser.Add('Gen.');
     FGreetingParser.Add('President');
     FGreetingParser.Add('Sister');
     FGreetingParser.Add('Father');
     FGreetingParser.Add('Brother');

     {Create default Suffix list}
     FSuffixParser.Clear;
     FSuffixParser.Add('PC.');
     FSuffixParser.Add('P.C.');
     FSuffixParser.Add('CFA');
     FSuffixParser.Add('Esq');
     FSuffixParser.Add('CFO');
     FSuffixParser.Add('CEO');
     FSuffixParser.Add('CPA');
     FSuffixParser.Add('MBA');
     FSuffixParser.Add('Phd');
     FSuffixParser.Add('MD');
     FSuffixParser.Add('DC');
     FSuffixParser.Add('Sr.');
     FSuffixParser.Add('Jr.');
     FSuffixParser.Add('II');
     FSuffixParser.Add('III');
end;
 
 
{================================================================================}

procedure TNameEdit.SetAbout(Value:TStringList);
begin
  if FAbout <> Value then
     FAbout.Assign( Value );
     {Create default greeting list}
     MessageBeep(0);
     FAbout.Clear;
     FAbout.Add('TNameEdit - Version 1.2');
     FAbout.Add('-----------------------------------------');
     FAbout.Add(' ');
     FAbout.Add('This component is distributed as freeware.');
     FAbout.Add(' ');
     FAbout.Add('E-Mail comments or questions to: jwood@itp-cti.com');
end;

{================================================================================}

procedure TNameEdit.SetGreetings(Value:TStringList);
begin
  if FGreetingParser <> Value then
     FGreetingParser.Assign( Value )
end;

{================================================================================}

procedure TNameEdit.SetSuffixs(Value:TStringList);
begin
  if FSuffixParser <> Value then
     FSuffixParser.Assign( Value )
end;

{================================================================================}

destructor TNameEdit.Destroy;
begin
  FGreetingParser.Free;
  FSuffixParser.Free;
  inherited Destroy;
end;
end.

